start_time_AQMS <- Sys.time()

# Loop Ordinary Kriging #######################################################

print("Loop Ordinary Kriging")

save_OK_result              <- paste0(save_result,"OK_monthly_",pollutant, "_",read_year)
model_raw_at_AQMS           <- matrix(NA,ncol=nrow(local_stations@data),nrow=0) #we will have a list with a data.frame for each hour
colnames(model_raw_at_AQMS) <- local_stations$ref_obs #matrix with empty data and the name of stations


# if set_model is FALSE, figure out which are the available_dates_model
if(set_model=="FALSE" & model_time_resolution=="daily"){
  available_dates_model <- daily_dates_format[!is.na(model[1,1,])]
}else if(set_model=="FALSE" & model_time_resolution=="hourly"){
  available_dates_model <- df_obs_sub$date[!is.na(model[1,1,])]
}

all_dates <- df_obs_sub$date
# for(i in 1:length(available_dates_model)){
for(i in 1:length(all_dates)){
  # i <- 101
  # date_i <- available_dates_model[i]
  date_i <- all_dates[i]
  
  # if(i!=length(available_dates_model)){
  #   date_ii <- available_dates_model[i+1]
  if(i!=length(all_dates)){
    date_ii <- all_dates[i+1]
  }else{
    date_ii <- date_i
  }
  print(date_i)

  df_model <- model[,,i]
  
  #if all the model for a certain hour is nan we cannot apply UK
  if (!any(is.na(df_model))){
    
    # 1. Interpolate model values at obs locations
    #    1.1. Set spatialpointsdataframe
    #raw model:
    df_model                  <- data.frame(lat=lat_model,lon=lon_model,pollutant=sapply(df_model,as.matrix))
    coordinates(df_model)     <- ~lon+lat
    sp::proj4string(df_model) <- crs_latlon
    
    #config the coordinates in UTM
    if (compute_in_UTM) {
      df_model <- spTransform(df_model, sp::CRS(crs_utm))
    }
    
    #observations:
    df_obs                   <- data.frame(lat=local_stations@coords[,2],
                                           lon=local_stations@coords[,1],
                                           pollutant=NA,
                                           code=local_stations$ref_obs)
    coordinates(df_obs)      <- ~lon+lat
    sp::proj4string(df_obs)  <- crs_latlon
    
    #config the coordinates in UTM
    if (compute_in_UTM) {
      df_obs <- spTransform(df_obs, sp::CRS(crs_utm))
    }
    
    #    1.2 Compute Variogram
    set.seed(1) 
    pick_random_points <- sample(1:nrow(df_model),1000)
    
    pollutant.fit_ok <- automap::autofitVariogram(formula=pollutant~1, 
                                                 input_data=df_model[pick_random_points,],
                                                 fix.values=c(0,NA,NA),
                                                 miscFitOptions = list(min.np.bin=1)) #nugget,range,sill
    
    #Save the first variogram executed for a certain pollutant if we already don't have one saved:
     if (!file.exists(variogram_ok)) {
       saveRDS(pollutant.fit_ok,variogram_ok)
     }
    #    1.3 Compute ordinary Kriging
    
    # interpolate model to the coordinates of observations
    ok_gstat <- gstat(formula=pollutant~1, data = df_model, model = pollutant.fit_ok$var_model, nmax=100)
    
    # Interpolation of the model
    OK       <- predict(object=ok_gstat, newdata=df_obs)
    
    # !!forcing concentrations to be positive
    OK$var1.pred[OK$var1.pred<0] <- 0
    
    #If the variogram has singular matrix then it produces NAN's, so in that case we take a good Variogram instead:
    if(any(is.na(OK@data$var1.pred))){
      print("WARNING: VARIOGRAM DIVERGED! Using a precalculated VARIOGRAM")
      pollutant.fit_ok=readRDS(variogram_ok)
      ok_gstat <- gstat(formula=pollutant~1, data = df_model, model = pollutant.fit_ok$var_model, nmax=100)
      
      # Interpolation of the model
      OK <-predict(object=ok_gstat, newdata=df_obs)
    }
    
    model_raw_at_AQMS_h           <- matrix(data=OK$var1.pred, nrow=1)
    colnames(model_raw_at_AQMS_h) <- df_obs$code
    
    model_raw_at_AQMS <-  plyr::rbind.fill(as.data.frame(model_raw_at_AQMS), as.data.frame(model_raw_at_AQMS_h))
    
  } else{
    print(paste0("WARNING: The model is corrupted or non-existent in ", date_i))
    model_raw_at_AQMS_h           <- matrix(data=NA, nrow=1,ncol=nrow(local_stations@data))
    colnames(model_raw_at_AQMS_h) <- local_stations@data$ref_obs
    model_raw_at_AQMS             <- plyr::rbind.fill(as.data.frame(model_raw_at_AQMS),as.data.frame(model_raw_at_AQMS_h))
  }
  
  # Check if we need to save the OK (last day of each month)
  if (!file.exists(save_OK_result)) {
    dir.create(save_OK_result)
  }
  
  # month_num <- mes[hour]
  if(time_resolution=="hourly"){
    # available_dates_model <- available_dates_model %>% 
    #   as.POSIXct("%Y%m%d %H:%M",tz="UTC")
    all_dates <- all_dates %>%
      as.Date("%Y%m%d",tz="UTC")
    
    month_num <- date_i %>%
      as.Date("%Y%m%d %H:%M") %>%
      month()
    next_month_num <- date_ii %>%
      as.Date("%Y%m%d %H:%M") %>%
      month()
    # day_num <- date_i %>%
    #   as.Date("%Y%m%d %H:%M") %>%
    #   day()
  }else if(time_resolution=="daily"){
    # available_dates_model <- available_dates_model %>% 
    #   as.Date("%Y%m%d",tz="UTC")
    
    all_dates <- all_dates %>%
      as.Date("%Y%m%d",tz="UTC")
    
    month_num <- date_i %>%
      as.Date("%Y%m%d") %>%
      month()
    next_month_num <- date_ii %>%
      as.Date("%Y%m%d") %>%
      month()
    
    # day_num <- date_i %>%
    #   as.Date("%Y%m%d") %>%
    #   day()
  }
  
  
  # if the month of next date is different, save
  # if(next_month_num != month_num | i==length(available_dates_model)){
  if(next_month_num != month_num | i==length(all_dates)){
      save(object=model_raw_at_AQMS, file= paste0(save_OK_result,"/model_raw_at_AQMS_", "month_", month_num, ".rds"))
  }
  
}


# model_raw_at_AQMS_df      <- cbind(date=available_dates_model, 
model_raw_at_AQMS_df      <- cbind(date=all_dates, 
                                   as.data.frame(model_raw_at_AQMS))

# impose timezone UTC (it was assuming CET)
tz(model_raw_at_AQMS_df$date) <- "UTC"


if (save_OK){
  saveRDS(object=model_raw_at_AQMS_df, file=paste0(save_result,"model_raw_at_AQMS_",file_format, "_",pollutant,"_df.rds"))
  print("model_raw_at_AQMS saved")
} else {
  print("model_raw_at_AQMS have not been saved")
}


end_time_AQMS    <- Sys.time()
temps_total_AQMS <- end_time_AQMS - start_time_AQMS
print(paste0("Interpolation of the model to AQMS, execution time:",temps_total_AQMS))

