# interpolation of observations ----
df_obs2 <- df_obs %>% 
  pivot_longer(cols=starts_with("ES"),names_to="code") %>%
  na.omit()

combined_data <- merge(sp_obs_locations, df_obs2, by = "code", all.x=F, all.y=T)

mapview(combined_data, zcol="value")



# Variogram fitting (weights of each station)
pollutant.fit_ok <- automap::autofitVariogram(formula=value~1, input_data=combined_data,model="Gau")

# Visualize the resulting variogram
plot(pollutant.fit_ok)

# Set gstat object  (pollutant~1 lineal estimation)                   
ok_gstat <- gstat(formula=pollutant_model~1, data = sp_model, model = pollutant.fit_ok$var_model, nmax=25)  # transform the variogram to a gstat (geostatistics)

# Interpolation using Ordinary Kriging
interpolated_obs <- predict(object=ok_gstat, newdata=sp_model)

mapview(interpolated_obs, zcol="var1.pred")

# get the interpolated values at the AQMS
# project the spatial points dataframe to indicate where to interpolate in UTM
sp_obs <- spTransform(sp_obs_locations, sp::CRS(crs_utm))
sp_obs <- spTransform(combined_data, sp::CRS(crs_utm))  
interpolated_obs_at_AQMS <- predict(object=ok_gstat, newdata=sp_obs)
mapview(interpolated_obs_at_AQMS, zcol="var1.pred")

saveRDS(interpolated_obs_at_AQMS, outputh_path)